@@ -64,6 +64,34 @@ def validate(cls, model):
         raise ImproperlyConfigured("'%s.list_per_page' should be a integer."
                 % cls.__name__)
 
+    # list_editable
+    if hasattr(cls, 'list_editable') and cls.list_editable:
+        check_isseq(cls, 'list_editable', cls.list_editable)
+        for idx, field_name in enumerate(cls.list_editable):
+            try:
+                field = opts.get_field_by_name(field_name)[0]
+            except models.FieldDoesNotExist:
+                raise ImproperlyConfigured("'%s.list_editable[%d]' refers to a "
+                    "field, '%s', not defined on %s."
+                    % (cls.__name__, idx, field_name, model.__name__))
+            if field_name not in cls.list_display:
+                raise ImproperlyConfigured("'%s.list_editable[%d]' refers to "
+                    "'%s' which is not defined in 'list_display'."
+                    % (cls.__name__, idx, field_name))
+            if field_name in cls.list_display_links:
+                raise ImproperlyConfigured("'%s' cannot be in both '%s.list_editable'"
+                    " and '%s.list_display_links'"
+                    % (field_name, cls.__name__, cls.__name__))
+            if not cls.list_display_links and cls.list_display[0] in cls.list_editable:
+                raise ImproperlyConfigured("'%s.list_editable[%d]' refers to"
+                    " the first field in list_display, '%s', which can't be"
+                    " used unless list_display_links is set."
+                    % (cls.__name__, idx, cls.list_display[0]))
+            if not field.editable:
+                raise ImproperlyConfigured("'%s.list_editable[%d]' refers to a "
+                    "field, '%s', which isn't editable through the admin."
+                    % (cls.__name__, idx, field_name))
+
     # search_fields = ()
     if hasattr(cls, 'search_fields'):
         check_isseq(cls, 'search_fields', cls.search_fields)
@@ -103,6 +131,7 @@ def validate(cls, model):
             raise ImproperlyConfigured("'%s.%s' should be a boolean."
                     % (cls.__name__, attr))
 
+
     # inlines = []
     if hasattr(cls, 'inlines'):
         check_isseq(cls, 'inlines', cls.inlines)
@@ -120,12 +149,16 @@ def validate(cls, model):
             validate_inline(inline, cls, model)
 
 def validate_inline(cls, parent, parent_model):
+    
     # model is already verified to exist and be a Model
     if cls.fk_name: # default value is None
         f = get_field(cls, cls.model, cls.model._meta, 'fk_name', cls.fk_name)
         if not isinstance(f, models.ForeignKey):
             raise ImproperlyConfigured("'%s.fk_name is not an instance of "
                     "models.ForeignKey." % cls.__name__)
+
+    fk = _get_foreign_key(parent_model, cls.model, fk_name=cls.fk_name, can_fail=True)
+
     # extra = 3
     # max_num = 0
     for attr in ('extra', 'max_num'):
@@ -140,7 +173,6 @@ def validate_inline(cls, parent, parent_model):
 
     # exclude
     if hasattr(cls, 'exclude') and cls.exclude:
-        fk = _get_foreign_key(parent_model, cls.model, can_fail=True)
         if fk and fk.name in cls.exclude:
             raise ImproperlyConfigured("%s cannot exclude the field "
                     "'%s' - this is the foreign key to the parent model "
@@ -188,6 +220,20 @@ def validate_base(cls, model):
         for field in flattened_fieldsets:
             check_formfield(cls, model, opts, "fieldsets[%d][1]['fields']" % idx, field)
 
+    # exclude
+    if cls.exclude: # default value is None
+        check_isseq(cls, 'exclude', cls.exclude)
+        for field in cls.exclude:
+            check_formfield(cls, model, opts, 'exclude', field)
+            try:
+                f = opts.get_field(field)
+            except models.FieldDoesNotExist:
+                # If we can't find a field on the model that matches,
+                # it could be an extra field on the form.
+                continue
+        if len(cls.exclude) > len(set(cls.exclude)):
+            raise ImproperlyConfigured('There are duplicate field(s) in %s.exclude' % cls.__name__)
+
     # form
     if hasattr(cls, 'form') and not issubclass(cls.form, BaseModelForm):
         raise ImproperlyConfigured("%s.form does not inherit from "
