@@ -116,7 +116,6 @@ from django.contrib.gis.gdal import CoordTransform, DataSource, \
     OGRException, OGRGeometry, OGRGeomType, SpatialReference
 from django.contrib.gis.gdal.field import \
     OFTDate, OFTDateTime, OFTInteger, OFTReal, OFTString, OFTTime
-from django.contrib.gis.models import GeometryColumns, SpatialRefSys
 from django.db import models, transaction
 from django.contrib.localflavor.us.models import USStateField
 
@@ -179,14 +178,20 @@ class LayerMapping(object):
             self.ds = data
         self.layer = self.ds[layer]
 
-        # Setting the mapping
+        # Setting the mapping & model attributes.
         self.mapping = mapping
-
-        # Setting the model, and getting the geometry column associated 
-        # with the model (an exception will be raised if there is no 
-        # geometry column).
         self.model = model
-        self.geo_col = self.geometry_column()
+ 
+        # Checking the layer -- intitialization of the object will fail if
+        # things don't check out before hand.
+        self.check_layer()
+
+        # Getting the geometry column associated with the model (an 
+        # exception will be raised if there is no geometry column).
+        if SpatialBackend.mysql:
+            transform = False
+        else:
+            self.geo_col = self.geometry_column()
 
         # Checking the source spatial reference system, and getting
         # the coordinate transformation object (unless the `transform`
@@ -197,10 +202,6 @@ class LayerMapping(object):
         else:
             self.transform = transform
 
-        # Checking the layer -- intitialization of the object will fail if
-        # things don't check out before hand.
-        self.check_layer()
-
         # Setting the encoding for OFTString fields, if specified.
         if encoding:
             # Making sure the encoding exists, if not a LookupError
@@ -246,7 +247,8 @@ class LayerMapping(object):
         there is no need to increment through each feature in the Layer.
         """
         # The geometry field of the model is set here.
-        # TODO: Support more than one geometry field / model.
+        # TODO: Support more than one geometry field / model.  However, this
+        # depends on the GDAL Driver in use.
         self.geom_field = False
         self.fields = {}
 
@@ -288,7 +290,8 @@ class LayerMapping(object):
                 # Making sure that the OGR Layer's Geometry is compatible.
                 ltype = self.layer.geom_type
                 if not (gtype == ltype or self.make_multi(ltype, model_field)):
-                    raise LayerMapError('Invalid mapping geometry; model has %s, feature has %s.' % (fld_name, gtype))
+                    raise LayerMapError('Invalid mapping geometry; model has %s, '
+                                        'layer geometry type is %s.' % (fld_name, ltype))
 
                 # Setting the `geom_field` attribute w/the name of the model field
                 # that is a Geometry.
@@ -327,6 +330,7 @@ class LayerMapping(object):
 
     def check_srs(self, source_srs):
         "Checks the compatibility of the given spatial reference object."
+        from django.contrib.gis.models import SpatialRefSys
         if isinstance(source_srs, SpatialReference):
             sr = source_srs
         elif isinstance(source_srs, SpatialRefSys):
@@ -498,6 +502,7 @@ class LayerMapping(object):
     #### Other model methods ####
     def coord_transform(self):
         "Returns the coordinate transformation object."
+        from django.contrib.gis.models import SpatialRefSys
         try:
             # Getting the target spatial reference system
             target_srs = SpatialRefSys.objects.get(srid=self.geo_col.srid).srs
@@ -509,11 +514,28 @@ class LayerMapping(object):
 
     def geometry_column(self):
         "Returns the GeometryColumn model associated with the geographic column."
-        # Getting the GeometryColumn object.
+        from django.contrib.gis.models import GeometryColumns
+        # Use the `get_field_by_name` on the model's options so that we
+        # get the correct model if there's model inheritance -- otherwise
+        # the returned model is None.
+        opts = self.model._meta
+        fld, model, direct, m2m = opts.get_field_by_name(self.geom_field)
+        if model is None: model = self.model
+
+        # Trying to get the `GeometryColumns` object that corresponds to the
+        # the geometry field.
         try:
-            db_table = self.model._meta.db_table
-            if SpatialBackend.name == 'oracle': db_table = db_table.upper()
-            gc_kwargs = {GeometryColumns.table_name_col() : db_table}
+            db_table = model._meta.db_table
+            geo_col = fld.column
+
+            if SpatialBackend.oracle:
+                # Making upper case for Oracle.
+                db_table = db_table.upper()
+                geo_col = geo_col.upper()
+
+            gc_kwargs = { GeometryColumns.table_name_col() : db_table,
+                          GeometryColumns.geom_col_name() : geo_col,
+                         }
             return GeometryColumns.objects.get(**gc_kwargs)
         except Exception, msg:
             raise LayerMapError('Geometry column does not exist for model. (did you run syncdb?):\n %s' % msg)
