@@ -13,7 +13,7 @@ from email.MIMEText import MIMEText
 from email.MIMEMultipart import MIMEMultipart
 from email.MIMEBase import MIMEBase
 from email.Header import Header
-from email.Utils import formatdate, parseaddr, formataddr
+from email.Utils import formatdate, getaddresses, formataddr
 
 from django.conf import settings
 from django.utils.encoding import smart_str, force_unicode
@@ -69,8 +69,9 @@ def make_msgid(idstring=None):
 class BadHeaderError(ValueError):
     pass
 
-def forbid_multi_line_headers(name, val):
+def forbid_multi_line_headers(name, val, encoding):
     """Forbids multi-line headers, to prevent header injection."""
+    encoding = encoding or settings.DEFAULT_CHARSET
     val = force_unicode(val)
     if '\n' in val or '\r' in val:
         raise BadHeaderError("Header values can't contain newlines (got %r for header %r)" % (val, name))
@@ -79,26 +80,35 @@ def forbid_multi_line_headers(name, val):
     except UnicodeEncodeError:
         if name.lower() in ('to', 'from', 'cc'):
             result = []
-            for item in val.split(', '):
-                nm, addr = parseaddr(item)
-                nm = str(Header(nm, settings.DEFAULT_CHARSET))
+            for nm, addr in getaddresses((val,)):
+                nm = str(Header(nm.encode(encoding), encoding))
                 result.append(formataddr((nm, str(addr))))
             val = ', '.join(result)
         else:
-            val = Header(val, settings.DEFAULT_CHARSET)
+            val = Header(val.encode(encoding), encoding)
     else:
         if name.lower() == 'subject':
             val = Header(val)
     return name, val
 
 class SafeMIMEText(MIMEText):
+
+    def __init__(self, text, subtype, charset):
+        self.encoding = charset
+        MIMEText.__init__(self, text, subtype, charset)
+
     def __setitem__(self, name, val):
-        name, val = forbid_multi_line_headers(name, val)
+        name, val = forbid_multi_line_headers(name, val, self.encoding)
         MIMEText.__setitem__(self, name, val)
 
 class SafeMIMEMultipart(MIMEMultipart):
+
+    def __init__(self, _subtype='mixed', boundary=None, _subparts=None, encoding=None, **_params):
+        self.encoding = encoding
+        MIMEMultipart.__init__(self, _subtype, boundary, _subparts, **_params)
+
     def __setitem__(self, name, val):
-        name, val = forbid_multi_line_headers(name, val)
+        name, val = forbid_multi_line_headers(name, val, self.encoding)
         MIMEMultipart.__setitem__(self, name, val)
 
 class SMTPConnection(object):
@@ -112,7 +122,10 @@ class SMTPConnection(object):
         self.port = port or settings.EMAIL_PORT
         self.username = username or settings.EMAIL_HOST_USER
         self.password = password or settings.EMAIL_HOST_PASSWORD
-        self.use_tls = (use_tls is not None) and use_tls or settings.EMAIL_USE_TLS
+        if use_tls is None:
+            self.use_tls = settings.EMAIL_USE_TLS
+        else:
+            self.use_tls = use_tls
         self.fail_silently = fail_silently
         self.connection = None
 
@@ -232,11 +245,11 @@ class EmailMessage(object):
 
     def message(self):
         encoding = self.encoding or settings.DEFAULT_CHARSET
-        msg = SafeMIMEText(smart_str(self.body, settings.DEFAULT_CHARSET),
+        msg = SafeMIMEText(smart_str(self.body, encoding),
                            self.content_subtype, encoding)
         msg = self._create_message(msg)
         msg['Subject'] = self.subject
-        msg['From'] = self.extra_headers.pop('From', self.from_email)
+        msg['From'] = self.extra_headers.get('From', self.from_email)
         msg['To'] = ', '.join(self.to)
 
         # Email header names are case-insensitive (RFC 2045), so we have to
@@ -247,6 +260,8 @@ class EmailMessage(object):
         if 'message-id' not in header_names:
             msg['Message-ID'] = make_msgid()
         for name, value in self.extra_headers.items():
+            if name.lower() == 'from':
+                continue
             msg[name] = value
         return msg
 
@@ -259,6 +274,10 @@ class EmailMessage(object):
 
     def send(self, fail_silently=False):
         """Sends the email message."""
+        if not self.recipients():
+            # Don't bother creating the network connection if there's nobody to
+            # send to.
+            return 0
         return self.get_connection(fail_silently).send_messages([self])
 
     def attach(self, filename=None, content=None, mimetype=None):
@@ -287,8 +306,9 @@ class EmailMessage(object):
 
     def _create_attachments(self, msg):
         if self.attachments:
+            encoding = self.encoding or settings.DEFAULT_CHARSET
             body_msg = msg
-            msg = SafeMIMEMultipart(_subtype=self.mixed_subtype)
+            msg = SafeMIMEMultipart(_subtype=self.mixed_subtype, encoding=encoding)
             if self.body:
                 msg.attach(body_msg)
             for attachment in self.attachments:
@@ -304,8 +324,8 @@ class EmailMessage(object):
         """
         basetype, subtype = mimetype.split('/', 1)
         if basetype == 'text':
-            attachment = SafeMIMEText(smart_str(content,
-                settings.DEFAULT_CHARSET), subtype, settings.DEFAULT_CHARSET)
+            encoding = self.encoding or settings.DEFAULT_CHARSET
+            attachment = SafeMIMEText(smart_str(content, encoding), subtype, encoding)
         else:
             # Encode non-text attachments with base64.
             attachment = MIMEBase(basetype, subtype)
@@ -359,9 +379,10 @@ class EmailMultiAlternatives(EmailMessage):
         return self._create_attachments(self._create_alternatives(msg))
 
     def _create_alternatives(self, msg):
+        encoding = self.encoding or settings.DEFAULT_CHARSET
         if self.alternatives:
             body_msg = msg
-            msg = SafeMIMEMultipart(_subtype=self.alternative_subtype)
+            msg = SafeMIMEMultipart(_subtype=self.alternative_subtype, encoding=encoding)
             if self.body:
                 msg.attach(body_msg)
             for alternative in self.alternatives:
@@ -407,12 +428,16 @@ def send_mass_mail(datatuple, fail_silently=False, auth_user=None,
 
 def mail_admins(subject, message, fail_silently=False):
     """Sends a message to the admins, as defined by the ADMINS setting."""
+    if not settings.ADMINS:
+        return
     EmailMessage(settings.EMAIL_SUBJECT_PREFIX + subject, message,
                  settings.SERVER_EMAIL, [a[1] for a in settings.ADMINS]
                  ).send(fail_silently=fail_silently)
 
 def mail_managers(subject, message, fail_silently=False):
     """Sends a message to the managers, as defined by the MANAGERS setting."""
+    if not settings.MANAGERS:
+        return
     EmailMessage(settings.EMAIL_SUBJECT_PREFIX + subject, message,
                  settings.SERVER_EMAIL, [a[1] for a in settings.MANAGERS]
                  ).send(fail_silently=fail_silently)
