@@ -141,15 +141,22 @@ class RelatedField(object):
                     v = v[0]
             return v
 
-        if hasattr(value, 'as_sql'):
-            sql, params = value.as_sql()
+        if hasattr(value, 'as_sql') or hasattr(value, '_as_sql'):
+            # If the value has a relabel_aliases method, it will need to
+            # be invoked before the final SQL is evaluated
+            if hasattr(value, 'relabel_aliases'):
+                return value
+            if hasattr(value, 'as_sql'):
+                sql, params = value.as_sql()
+            else:
+                sql, params = value._as_sql()
             return QueryWrapper(('(%s)' % sql), params)
 
         # FIXME: lt and gt are explicitally allowed to make
         # get_(next/prev)_by_date work; other lookups are not allowed since that
         # gets messy pretty quick. This is a good candidate for some refactoring
         # in the future.
-        if lookup_type in ['exact', 'gt', 'lt']:
+        if lookup_type in ['exact', 'gt', 'lt', 'gte', 'lte']:
             return [pk_trace(value)]
         if lookup_type in ('range', 'in'):
             return [pk_trace(v) for v in value]
@@ -599,7 +606,7 @@ class ReverseManyRelatedObjectsDescriptor(object):
             model=rel_model,
             core_filters={'%s__pk' % self.field.related_query_name(): instance._get_pk_val()},
             instance=instance,
-            symmetrical=(self.field.rel.symmetrical and isinstance(instance, rel_model)),
+            symmetrical=self.field.rel.symmetrical,
             join_table=qn(self.field.m2m_db_table()),
             source_col_name=qn(self.field.m2m_column_name()),
             target_col_name=qn(self.field.m2m_reverse_name())
@@ -676,6 +683,7 @@ class ManyToManyRel(object):
 
 class ForeignKey(RelatedField, Field):
     empty_strings_allowed = False
+    description = ugettext_lazy("Foreign Key (type determined by related field)")
     def __init__(self, to, to_field=None, rel_class=ManyToOneRel, **kwargs):
         try:
             to_name = to._meta.object_name.lower()
@@ -770,6 +778,7 @@ class OneToOneField(ForeignKey):
     always returns the object pointed to (since there will only ever be one),
     rather than returning a list.
     """
+    description = ugettext_lazy("One-to-one relationship")
     def __init__(self, to, to_field=None, **kwargs):
         kwargs['unique'] = True
         super(OneToOneField, self).__init__(to, to_field, OneToOneRel, **kwargs)
@@ -777,8 +786,6 @@ class OneToOneField(ForeignKey):
     def contribute_to_related_class(self, cls, related):
         setattr(cls, related.get_accessor_name(),
                 SingleRelatedObjectDescriptor(related))
-        if not cls._meta.one_to_one_field:
-            cls._meta.one_to_one_field = self
 
     def formfield(self, **kwargs):
         if self.rel.parent_link:
@@ -786,6 +793,7 @@ class OneToOneField(ForeignKey):
         return super(OneToOneField, self).formfield(**kwargs)
 
 class ManyToManyField(RelatedField, Field):
+    description = ugettext_lazy("Many-to-many relationship")
     def __init__(self, to, **kwargs):
         try:
             assert not to._meta.abstract, "%s cannot define a relation with abstract class %s" % (self.__class__.__name__, to._meta.object_name)
@@ -796,7 +804,7 @@ class ManyToManyField(RelatedField, Field):
         kwargs['rel'] = ManyToManyRel(to,
             related_name=kwargs.pop('related_name', None),
             limit_choices_to=kwargs.pop('limit_choices_to', None),
-            symmetrical=kwargs.pop('symmetrical', True),
+            symmetrical=kwargs.pop('symmetrical', to==RECURSIVE_RELATIONSHIP_CONSTANT),
             through=kwargs.pop('through', None))
 
         self.db_table = kwargs.pop('db_table', None)
@@ -835,7 +843,7 @@ class ManyToManyField(RelatedField, Field):
                         self._m2m_column_name_cache = f.column
                         break
             # If this is an m2m relation to self, avoid the inevitable name clash
-            elif related.model == related.parent_model:
+            elif related.model._meta.object_name.lower() == related.parent_model._meta.object_name.lower():
                 self._m2m_column_name_cache = 'from_' + related.model._meta.object_name.lower() + '_id'
             else:
                 self._m2m_column_name_cache = related.model._meta.object_name.lower() + '_id'
