@@ -5,11 +5,12 @@ Query subclasses which provide extra functionality beyond simple data retrieval.
 from django.core.exceptions import FieldError
 from django.db.models.sql.constants import *
 from django.db.models.sql.datastructures import Date
+from django.db.models.sql.expressions import SQLEvaluator
 from django.db.models.sql.query import Query
-from django.db.models.sql.where import AND
+from django.db.models.sql.where import AND, Constraint
 
 __all__ = ['DeleteQuery', 'UpdateQuery', 'InsertQuery', 'DateQuery',
-        'CountQuery']
+        'AggregateQuery']
 
 class DeleteQuery(Query):
     """
@@ -48,23 +49,28 @@ class DeleteQuery(Query):
             if not isinstance(related.field, generic.GenericRelation):
                 for offset in range(0, len(pk_list), GET_ITERATOR_CHUNK_SIZE):
                     where = self.where_class()
-                    where.add((None, related.field.m2m_reverse_name(),
-                            related.field, 'in',
+                    where.add((Constraint(None,
+                            related.field.m2m_reverse_name(), related.field),
+                            'in',
                             pk_list[offset : offset+GET_ITERATOR_CHUNK_SIZE]),
                             AND)
                     self.do_query(related.field.m2m_db_table(), where)
 
         for f in cls._meta.many_to_many:
             w1 = self.where_class()
+            db_prep_value = None
             if isinstance(f, generic.GenericRelation):
                 from django.contrib.contenttypes.models import ContentType
-                field = f.rel.to._meta.get_field(f.content_type_field_name)
-                w1.add((None, field.column, field, 'exact',
+                ct_field = f.rel.to._meta.get_field(f.content_type_field_name)
+                w1.add((Constraint(None, ct_field.column, ct_field), 'exact',
                         ContentType.objects.get_for_model(cls).id), AND)
+                id_field = f.rel.to._meta.get_field(f.object_id_field_name)
+                db_prep_value = id_field.get_db_prep_value
             for offset in range(0, len(pk_list), GET_ITERATOR_CHUNK_SIZE):
                 where = self.where_class()
-                where.add((None, f.m2m_column_name(), f, 'in',
-                        pk_list[offset : offset + GET_ITERATOR_CHUNK_SIZE]),
+                where.add((Constraint(None, f.m2m_column_name(), f), 'in',
+                        map(db_prep_value,
+                            pk_list[offset : offset + GET_ITERATOR_CHUNK_SIZE])),
                         AND)
                 if w1:
                     where.add(w1, AND)
@@ -81,7 +87,7 @@ class DeleteQuery(Query):
         for offset in range(0, len(pk_list), GET_ITERATOR_CHUNK_SIZE):
             where = self.where_class()
             field = self.model._meta.pk
-            where.add((None, field.column, field, 'in',
+            where.add((Constraint(None, field.column, field), 'in',
                     pk_list[offset : offset + GET_ITERATOR_CHUNK_SIZE]), AND)
             self.do_query(self.model._meta.db_table, where)
 
@@ -140,7 +146,11 @@ class UpdateQuery(Query):
         result.append('SET')
         values, update_params = [], []
         for name, val, placeholder in self.values:
-            if val is not None:
+            if hasattr(val, 'as_sql'):
+                sql, params = val.as_sql(qn)
+                values.append('%s = %s' % (qn(name), sql))
+                update_params.extend(params)
+            elif val is not None:
                 values.append('%s = %s' % (qn(name), placeholder))
                 update_params.append(val)
             else:
@@ -172,7 +182,7 @@ class UpdateQuery(Query):
         # from other tables.
         query = self.clone(klass=Query)
         query.bump_prefix()
-        query.extra_select = {}
+        query.extra = {}
         query.select = []
         query.add_fields([query.model._meta.pk.name])
         must_pre_select = count > 1 and not self.connection.features.update_can_self_select
@@ -205,7 +215,7 @@ class UpdateQuery(Query):
         for offset in range(0, len(pk_list), GET_ITERATOR_CHUNK_SIZE):
             self.where = self.where_class()
             f = self.model._meta.pk
-            self.where.add((None, f.column, f, 'in',
+            self.where.add((Constraint(None, f.column, f), 'in',
                     pk_list[offset : offset + GET_ITERATOR_CHUNK_SIZE]),
                     AND)
             self.values = [(related_field.column, None, '%s')]
@@ -244,6 +254,8 @@ class UpdateQuery(Query):
             else:
                 placeholder = '%s'
 
+            if hasattr(val, 'evaluate'):
+                val = SQLEvaluator(val, self, allow_joins=False)
             if model:
                 self.add_related_update(model, field.column, val, placeholder)
             else:
@@ -272,7 +284,7 @@ class UpdateQuery(Query):
         for model, values in self.related_updates.iteritems():
             query = UpdateQuery(model, self.connection)
             query.values = values
-            if self.related_ids:
+            if self.related_ids is not None:
                 query.add_filter(('pk__in', self.related_ids))
             result.append(query)
         return result
@@ -283,10 +295,11 @@ class InsertQuery(Query):
         self.columns = []
         self.values = []
         self.params = ()
+        self.return_id = False
 
     def clone(self, klass=None, **kwargs):
         extras = {'columns': self.columns[:], 'values': self.values[:],
-                'params': self.params}
+                  'params': self.params, 'return_id': self.return_id}
         extras.update(kwargs)
         return super(InsertQuery, self).clone(klass, **extras)
 
@@ -294,16 +307,27 @@ class InsertQuery(Query):
         # We don't need quote_name_unless_alias() here, since these are all
         # going to be column names (so we can avoid the extra overhead).
         qn = self.connection.ops.quote_name
-        result = ['INSERT INTO %s' % qn(self.model._meta.db_table)]
+        opts = self.model._meta
+        result = ['INSERT INTO %s' % qn(opts.db_table)]
         result.append('(%s)' % ', '.join([qn(c) for c in self.columns]))
         result.append('VALUES (%s)' % ', '.join(self.values))
-        return ' '.join(result), self.params
+        params = self.params
+        if self.return_id and self.connection.features.can_return_id_from_insert:
+            col = "%s.%s" % (qn(opts.db_table), qn(opts.pk.column))
+            r_fmt, r_params = self.connection.ops.return_insert_id()
+            result.append(r_fmt % col)
+            params = params + r_params
+        return ' '.join(result), params
 
     def execute_sql(self, return_id=False):
+        self.return_id = return_id
         cursor = super(InsertQuery, self).execute_sql(None)
-        if return_id and cursor:
-            return self.connection.ops.last_insert_id(cursor,
-                    self.model._meta.db_table, self.model._meta.pk.column)
+        if not (return_id and cursor):
+            return
+        if self.connection.features.can_return_id_from_insert:
+            return self.connection.ops.fetch_returned_insert_id(cursor)
+        return self.connection.ops.last_insert_id(cursor,
+                self.model._meta.db_table, self.model._meta.pk.column)
 
     def insert_values(self, insert_values, raw_values=False):
         """
@@ -389,19 +413,29 @@ class DateQuery(Query):
         self.select = [select]
         self.select_fields = [None]
         self.select_related = False # See #7097.
-        self.extra_select = {}
+        self.set_extra_mask([])
         self.distinct = True
         self.order_by = order == 'ASC' and [1] or [-1]
 
-class CountQuery(Query):
+class AggregateQuery(Query):
     """
-    A CountQuery knows how to take a normal query which would select over
-    multiple distinct columns and turn it into SQL that can be used on a
-    variety of backends (it requires a select in the FROM clause).
+    An AggregateQuery takes another query as a parameter to the FROM
+    clause and only selects the elements in the provided list.
     """
-    def get_from_clause(self):
-        result, params = self._query.as_sql()
-        return ['(%s) A1' % result], params
+    def add_subquery(self, query):
+        self.subquery, self.sub_params = query.as_sql(with_col_aliases=True)
 
-    def get_ordering(self):
-        return ()
+    def as_sql(self, quote_func=None):
+        """
+        Creates the SQL for this query. Returns the SQL string and list of
+        parameters.
+        """
+        sql = ('SELECT %s FROM (%s) subquery' % (
+            ', '.join([
+                aggregate.as_sql()
+                for aggregate in self.aggregate_select.values()
+            ]),
+            self.subquery)
+        )
+        params = self.sub_params
+        return (sql, params)
