@@ -2,7 +2,6 @@
 Field classes.
 """
 
-import copy
 import datetime
 import os
 import re
@@ -13,6 +12,11 @@ try:
 except ImportError:
     from StringIO import StringIO
 
+import django.core.exceptions
+import django.utils.copycompat as copy
+from django.utils.translation import ugettext_lazy as _
+from django.utils.encoding import smart_unicode, smart_str
+
 # Python 2.3 fallbacks
 try:
     from decimal import Decimal, DecimalException
@@ -23,13 +27,8 @@ try:
 except NameError:
     from sets import Set as set
 
-import django.core.exceptions
-from django.utils.translation import ugettext_lazy as _
-from django.utils.encoding import smart_unicode, smart_str
-
 from util import ErrorList, ValidationError
-from widgets import TextInput, PasswordInput, HiddenInput, MultipleHiddenInput, FileInput, CheckboxInput, Select, NullBooleanSelect, SelectMultiple, DateTimeInput, TimeInput, SplitDateTimeWidget, SplitHiddenDateTimeWidget
-from django.core.files.uploadedfile import SimpleUploadedFile as UploadedFile
+from widgets import TextInput, PasswordInput, HiddenInput, MultipleHiddenInput, FileInput, CheckboxInput, Select, NullBooleanSelect, SelectMultiple, DateInput, DateTimeInput, TimeInput, SplitDateTimeWidget, SplitHiddenDateTimeWidget
 
 __all__ = (
     'Field', 'CharField', 'IntegerField',
@@ -250,6 +249,12 @@ class DecimalField(Field):
         except DecimalException:
             raise ValidationError(self.error_messages['invalid'])
 
+        # Check for NaN, Inf and -Inf values. We can't compare directly for NaN,
+        # since it is never equal to itself. However, NaN is the only value that
+        # isn't equal to itself, so we can use this to identify NaN
+        if value != value or value == Decimal("Inf") or value == Decimal("-Inf"):
+            raise ValidationError(self.error_messages['invalid'])
+
         sign, digittuple, exponent = value.as_tuple()
         decimals = abs(exponent)
         # digittuple doesn't include any leading zeros.
@@ -283,6 +288,7 @@ DEFAULT_DATE_INPUT_FORMATS = (
 )
 
 class DateField(Field):
+    widget = DateInput
     default_error_messages = {
         'invalid': _(u'Enter a valid date.'),
     }
@@ -492,7 +498,12 @@ class ImageField(FileField):
             return None
         elif not data and initial:
             return initial
-        from PIL import Image
+
+        # Try to import PIL in either of the two ways it can end up installed.
+        try:
+            from PIL import Image
+        except ImportError:
+            import Image
 
         # We need to get a file object for PIL. We might have a path or we might
         # have to read the data into memory.
@@ -658,7 +669,7 @@ class ChoiceField(Field):
     def valid_value(self, value):
         "Check to see if the provided value is a valid choice"
         for k, v in self.choices:
-            if type(v) in (tuple, list):
+            if isinstance(v, (list, tuple)):
                 # This is an optgroup, so look inside the group for options
                 for k2, v2 in v:
                     if value == smart_unicode(k2):
@@ -863,13 +874,13 @@ class SplitDateTimeField(MultiValueField):
         'invalid_time': _(u'Enter a valid time.'),
     }
 
-    def __init__(self, *args, **kwargs):
+    def __init__(self, input_date_formats=None, input_time_formats=None, *args, **kwargs):
         errors = self.default_error_messages.copy()
         if 'error_messages' in kwargs:
             errors.update(kwargs['error_messages'])
         fields = (
-            DateField(error_messages={'invalid': errors['invalid_date']}),
-            TimeField(error_messages={'invalid': errors['invalid_time']}),
+            DateField(input_formats=input_date_formats, error_messages={'invalid': errors['invalid_date']}),
+            TimeField(input_formats=input_time_formats, error_messages={'invalid': errors['invalid_time']}),
         )
         super(SplitDateTimeField, self).__init__(fields, *args, **kwargs)
 
