@@ -37,9 +37,14 @@ class CommentNode(Node):
     def render(self, context):
         return ''
 
+class CsrfTokenNode(Node):
+    # This no-op tag exists to allow 1.1.X code to be compatible with Django 1.2
+    def render(self, context):
+        return u''
+
 class CycleNode(Node):
     def __init__(self, cyclevars, variable_name=None):
-        self.cycle_iter = itertools_cycle([Variable(v) for v in cyclevars])
+        self.cycle_iter = itertools_cycle(cyclevars)
         self.variable_name = variable_name
 
     def render(self, context):
@@ -70,23 +75,26 @@ class FilterNode(Node):
 
 class FirstOfNode(Node):
     def __init__(self, vars):
-        self.vars = map(Variable, vars)
+        self.vars = vars
 
     def render(self, context):
         for var in self.vars:
-            try:
-                value = var.resolve(context)
-            except VariableDoesNotExist:
-                continue
+            value = var.resolve(context, True)
             if value:
                 return smart_unicode(value)
         return u''
 
 class ForNode(Node):
-    def __init__(self, loopvars, sequence, is_reversed, nodelist_loop):
+    child_nodelists = ('nodelist_loop', 'nodelist_empty')
+
+    def __init__(self, loopvars, sequence, is_reversed, nodelist_loop, nodelist_empty=None):
         self.loopvars, self.sequence = loopvars, sequence
         self.is_reversed = is_reversed
         self.nodelist_loop = nodelist_loop
+        if nodelist_empty is None:
+            self.nodelist_empty = NodeList()
+        else:
+            self.nodelist_empty = nodelist_empty
 
     def __repr__(self):
         reversed_text = self.is_reversed and ' reversed' or ''
@@ -97,16 +105,10 @@ class ForNode(Node):
     def __iter__(self):
         for node in self.nodelist_loop:
             yield node
-
-    def get_nodes_by_type(self, nodetype):
-        nodes = []
-        if isinstance(self, nodetype):
-            nodes.append(self)
-        nodes.extend(self.nodelist_loop.get_nodes_by_type(nodetype))
-        return nodes
+        for node in self.nodelist_empty:
+            yield node
 
     def render(self, context):
-        nodelist = NodeList()
         if 'forloop' in context:
             parentloop = context['forloop']
         else:
@@ -121,6 +123,10 @@ class ForNode(Node):
         if not hasattr(values, '__len__'):
             values = list(values)
         len_values = len(values)
+        if len_values < 1:
+            context.pop()
+            return self.nodelist_empty.render(context)
+        nodelist = NodeList()
         if self.is_reversed:
             values = reversed(values)
         unpack = len(self.loopvars) > 1
@@ -157,10 +163,12 @@ class ForNode(Node):
         return nodelist.render(context)
 
 class IfChangedNode(Node):
+    child_nodelists = ('nodelist_true', 'nodelist_false')
+
     def __init__(self, nodelist_true, nodelist_false, *varlist):
         self.nodelist_true, self.nodelist_false = nodelist_true, nodelist_false
         self._last_seen = None
-        self._varlist = map(Variable, varlist)
+        self._varlist = varlist
         self._id = str(id(self))
 
     def render(self, context):
@@ -171,7 +179,7 @@ class IfChangedNode(Node):
             if self._varlist:
                 # Consider multiple parameters.  This automatically behaves
                 # like an OR evaluation of the multiple variables.
-                compare_to = [var.resolve(context) for var in self._varlist]
+                compare_to = [var.resolve(context, True) for var in self._varlist]
             else:
                 compare_to = self.nodelist_true.render(context)
         except VariableDoesNotExist:
@@ -180,18 +188,17 @@ class IfChangedNode(Node):
         if compare_to != self._last_seen:
             firstloop = (self._last_seen == None)
             self._last_seen = compare_to
-            context.push()
-            context['ifchanged'] = {'firstloop': firstloop}
             content = self.nodelist_true.render(context)
-            context.pop()
             return content
         elif self.nodelist_false:
             return self.nodelist_false.render(context)
         return ''
 
 class IfEqualNode(Node):
+    child_nodelists = ('nodelist_true', 'nodelist_false')
+
     def __init__(self, var1, var2, nodelist_true, nodelist_false, negate):
-        self.var1, self.var2 = Variable(var1), Variable(var2)
+        self.var1, self.var2 = var1, var2
         self.nodelist_true, self.nodelist_false = nodelist_true, nodelist_false
         self.negate = negate
 
@@ -199,19 +206,15 @@ class IfEqualNode(Node):
         return "<IfEqualNode>"
 
     def render(self, context):
-        try:
-            val1 = self.var1.resolve(context)
-        except VariableDoesNotExist:
-            val1 = None
-        try:
-            val2 = self.var2.resolve(context)
-        except VariableDoesNotExist:
-            val2 = None
+        val1 = self.var1.resolve(context, True)
+        val2 = self.var2.resolve(context, True)
         if (self.negate and val1 != val2) or (not self.negate and val1 == val2):
             return self.nodelist_true.render(context)
         return self.nodelist_false.render(context)
 
 class IfNode(Node):
+    child_nodelists = ('nodelist_true', 'nodelist_false')
+
     def __init__(self, bool_exprs, nodelist_true, nodelist_false, link_type):
         self.bool_exprs = bool_exprs
         self.nodelist_true, self.nodelist_false = nodelist_true, nodelist_false
@@ -226,14 +229,6 @@ class IfNode(Node):
         for node in self.nodelist_false:
             yield node
 
-    def get_nodes_by_type(self, nodetype):
-        nodes = []
-        if isinstance(self, nodetype):
-            nodes.append(self)
-        nodes.extend(self.nodelist_true.get_nodes_by_type(nodetype))
-        nodes.extend(self.nodelist_false.get_nodes_by_type(nodetype))
-        return nodes
-
     def render(self, context):
         if self.link_type == IfNode.LinkTypes.or_:
             for ifnot, bool_expr in self.bool_exprs:
@@ -362,25 +357,24 @@ class URLNode(Node):
         args = [arg.resolve(context) for arg in self.args]
         kwargs = dict([(smart_str(k,'ascii'), v.resolve(context))
                        for k, v in self.kwargs.items()])
-        
-        
+
         # Try to look up the URL twice: once given the view name, and again
-        # relative to what we guess is the "main" app. If they both fail, 
-        # re-raise the NoReverseMatch unless we're using the 
+        # relative to what we guess is the "main" app. If they both fail,
+        # re-raise the NoReverseMatch unless we're using the
         # {% url ... as var %} construct in which cause return nothing.
         url = ''
         try:
-            url = reverse(self.view_name, args=args, kwargs=kwargs)
+            url = reverse(self.view_name, args=args, kwargs=kwargs, current_app=context.current_app)
         except NoReverseMatch, e:
             if settings.SETTINGS_MODULE:
                 project_name = settings.SETTINGS_MODULE.split('.')[0]
                 try:
                     url = reverse(project_name + '.' + self.view_name,
-                              args=args, kwargs=kwargs)
+                              args=args, kwargs=kwargs, current_app=context.current_app)
                 except NoReverseMatch:
                     if self.asvar is None:
                         # Re-raise the original exception, not the one with
-                        # the path relative to the project. This makes a 
+                        # the path relative to the project. This makes a
                         # better error message.
                         raise e
             else:
@@ -403,12 +397,15 @@ class WidthRatioNode(Node):
         try:
             value = self.val_expr.resolve(context)
             maxvalue = self.max_expr.resolve(context)
+            max_width = int(self.max_width.resolve(context))
         except VariableDoesNotExist:
             return ''
+        except ValueError:
+            raise TemplateSyntaxError("widthratio final argument must be an number")
         try:
             value = float(value)
             maxvalue = float(maxvalue)
-            ratio = (value / maxvalue) * int(self.max_width)
+            ratio = (value / maxvalue) * max_width
         except (ValueError, ZeroDivisionError):
             return ''
         return str(int(round(ratio)))
@@ -512,15 +509,22 @@ def cycle(parser, token):
 
     if len(args) > 4 and args[-2] == 'as':
         name = args[-1]
-        node = CycleNode(args[1:-2], name)
+        values = [parser.compile_filter(arg) for arg in args[1:-2]]
+        node = CycleNode(values, name)
         if not hasattr(parser, '_namedCycleNodes'):
             parser._namedCycleNodes = {}
         parser._namedCycleNodes[name] = node
     else:
-        node = CycleNode(args[1:])
+        values = [parser.compile_filter(arg) for arg in args[1:]]
+        node = CycleNode(values)
     return node
 cycle = register.tag(cycle)
 
+def csrf_token(parser, token):
+    # This no-op tag exists to allow 1.1.X code to be compatible with Django 1.2
+    return CsrfTokenNode()
+register.tag(csrf_token)
+
 def debug(parser, token):
     """
     Outputs a whole load of debugging information, including the current
@@ -591,14 +595,14 @@ def firstof(parser, token):
 
         {% filter force_escape %}
             {% firstof var1 var2 var3 "fallback value" %}
-	{% endfilter %}
+        {% endfilter %}
 
     """
     bits = token.split_contents()[1:]
     if len(bits) < 1:
         raise TemplateSyntaxError("'firstof' statement requires at least one"
                                   " argument")
-    return FirstOfNode(bits)
+    return FirstOfNode([parser.compile_filter(bit) for bit in bits])
 firstof = register.tag(firstof)
 
 #@register.tag(name="for")
@@ -623,6 +627,30 @@ def do_for(parser, token):
             {{ key }}: {{ value }}
         {% endfor %}
 
+    The ``for`` tag can take an optional ``{% empty %}`` clause that will
+    be displayed if the given array is empty or could not be found::
+
+        <ul>
+          {% for athlete in athlete_list %}
+            <li>{{ athlete.name }}</li>
+          {% empty %}
+            <li>Sorry, no athletes in this list.</li>
+          {% endfor %}
+        <ul>
+
+    The above is equivalent to -- but shorter, cleaner, and possibly faster
+    than -- the following::
+
+        <ul>
+          {% if althete_list %}
+            {% for athlete in athlete_list %}
+              <li>{{ athlete.name }}</li>
+            {% endfor %}
+          {% else %}
+            <li>Sorry, no athletes in this list.</li>
+          {% endif %}
+        </ul>
+
     The for loop sets a number of variables available within the loop:
 
         ==========================  ================================================
@@ -659,9 +687,14 @@ def do_for(parser, token):
                                       " %s" % token.contents)
 
     sequence = parser.compile_filter(bits[in_index+1])
-    nodelist_loop = parser.parse(('endfor',))
-    parser.delete_first_token()
-    return ForNode(loopvars, sequence, is_reversed, nodelist_loop)
+    nodelist_loop = parser.parse(('empty', 'endfor',))
+    token = parser.next_token()
+    if token.contents == 'empty':
+        nodelist_empty = parser.parse(('endfor',))
+        parser.delete_first_token()
+    else:
+        nodelist_empty = None
+    return ForNode(loopvars, sequence, is_reversed, nodelist_loop, nodelist_empty)
 do_for = register.tag("for", do_for)
 
 def do_ifequal(parser, token, negate):
@@ -676,7 +709,9 @@ def do_ifequal(parser, token, negate):
         parser.delete_first_token()
     else:
         nodelist_false = NodeList()
-    return IfEqualNode(bits[1], bits[2], nodelist_true, nodelist_false, negate)
+    val1 = parser.compile_filter(bits[1])
+    val2 = parser.compile_filter(bits[2])
+    return IfEqualNode(val1, val2, nodelist_true, nodelist_false, negate)
 
 #@register.tag
 def ifequal(parser, token):
@@ -839,7 +874,8 @@ def ifchanged(parser, token):
         parser.delete_first_token()
     else:
         nodelist_false = NodeList()
-    return IfChangedNode(nodelist_true, nodelist_false, *bits[1:])
+    values = [parser.compile_filter(bit) for bit in bits[1:]]
+    return IfChangedNode(nodelist_true, nodelist_false, *values)
 ifchanged = register.tag(ifchanged)
 
 #@register.tag
@@ -1080,7 +1116,7 @@ def url(parser, token):
     args = []
     kwargs = {}
     asvar = None
-        
+
     if len(bits) > 2:
         bits = iter(bits[2:])
         for bit in bits:
@@ -1116,12 +1152,10 @@ def widthratio(parser, token):
     if len(bits) != 4:
         raise TemplateSyntaxError("widthratio takes three arguments")
     tag, this_value_expr, max_value_expr, max_width = bits
-    try:
-        max_width = int(max_width)
-    except ValueError:
-        raise TemplateSyntaxError("widthratio final argument must be an integer")
+
     return WidthRatioNode(parser.compile_filter(this_value_expr),
-                          parser.compile_filter(max_value_expr), max_width)
+                          parser.compile_filter(max_value_expr),
+                          parser.compile_filter(max_width))
 widthratio = register.tag(widthratio)
 
 #@register.tag
