@@ -1,18 +1,31 @@
 # -*- coding: utf-8 -*-
 import datetime
+import tempfile
+import shutil
 
 from django.db import models
 # Can't import as "forms" due to implementation details in the test suite (the
 # current file is called "forms" and is already imported).
 from django import forms as django_forms
+from django.core.files.storage import FileSystemStorage
+
+temp_storage_location = tempfile.mkdtemp()
+temp_storage = FileSystemStorage(location=temp_storage_location)
 
 class BoundaryModel(models.Model):
     positive_integer = models.PositiveIntegerField(null=True, blank=True)
 
+callable_default_value = 0
+def callable_default():
+    global callable_default_value
+    callable_default_value = callable_default_value + 1
+    return callable_default_value
+
 class Defaults(models.Model):
     name = models.CharField(max_length=255, default='class default value')
     def_date = models.DateField(default = datetime.date(1980, 1, 1))
     value = models.IntegerField(default=42)
+    callable_default = models.IntegerField(default=callable_default)
 
 class ChoiceModel(models.Model):
     """For ModelChoiceField and ModelMultipleChoiceField tests."""
@@ -30,7 +43,7 @@ class ChoiceFieldModel(models.Model):
                                default=lambda: ChoiceOptionModel.objects.all()[0])
 
 class FileModel(models.Model):
-    file = models.FileField(upload_to='/')
+    file = models.FileField(storage=temp_storage, upload_to='tests')
 
 class FileForm(django_forms.Form):
     file1 = django_forms.FileField()
@@ -47,6 +60,12 @@ True
 {'file1': <SimpleUploadedFile: 我隻氣墊船裝滿晒鱔.txt (text/plain)>}
 >>> m = FileModel.objects.create(file=f.cleaned_data['file1'])
 
+# It's enough that m gets created without error.  Preservation of the exotic name is checked
+# in a file_uploads test; it's hard to do that correctly with doctest's unicode issues. So
+# we create and then immediately delete m so as to not leave the exotically named file around
+# for shutil.rmtree (on Windows) to have trouble with later.
+>>> m.delete()
+
 # Boundary conditions on a PostitiveIntegerField #########################
 >>> class BoundaryForm(ModelForm):
 ...     class Meta:
@@ -73,6 +92,10 @@ u'class default value'
 datetime.date(1980, 1, 1)
 >>> DefaultsForm().fields['value'].initial
 42
+>>> r1 = DefaultsForm()['callable_default'].as_widget()
+>>> r2 = DefaultsForm()['callable_default'].as_widget()
+>>> r1 == r2
+False
 
 In a ModelForm that is passed an instance, the initial values come from the
 instance's values, not the model's defaults.
@@ -85,6 +108,26 @@ datetime.date(1969, 4, 4)
 >>> instance_form.initial['value']
 12
 
+>>> from django.forms import CharField
+>>> class ExcludingForm(ModelForm):
+...     name = CharField(max_length=255)
+...     class Meta:
+...         model = Defaults
+...         exclude = ['name', 'callable_default']
+>>> f = ExcludingForm({'name': u'Hello', 'value': 99, 'def_date': datetime.date(1999, 3, 2)})
+>>> f.is_valid()
+True
+>>> f.cleaned_data['name']
+u'Hello'
+>>> obj = f.save()
+>>> obj.name
+u'class default value'
+>>> obj.value
+99
+>>> obj.def_date
+datetime.date(1999, 3, 2)
+>>> shutil.rmtree(temp_storage_location)
+
 In a ModelForm with a ModelChoiceField, if the model's ForeignKey has blank=False and a default,
 no empty option is created (regression test for #10792).
 
